'''

 This script took origin from the Gimp-chat topic KDE4/Aero Blur Effect.
 The main and simple purpose of this script is:
 - let the user perform his/her selection(s)
   (if not, a default selection is applied by the script)
 - create on the remaining part of the image a glass effect,
   with different glass aspects
 - leave selected part of the image untouched,
   so that it looks like through "holes" in the glass
 ------------------------------------------------------------------
 current version is derived from the previous version 1.1 in Scheme
 all calls to G'MIC have been replaced by Gimp functions
 a new frame type has been added (Omg style)
 ------------------------------------------------------------------
 Flow implemented to get the final result:
 
 1. check whether the user did a selection: if not make one
    (a rounded rectangle)
 2. duplicate the input layer
 3. apply the chosen effect on the non-selected part of the image
 4. duplicate again the original layer
 5. apply on the non-selected part of the image a colour,
    derived from the average of the source image colours,
    merged with a glass texture
 6. apply a bevelling effect (if so chosen by the user)
 7. flatten the image (if so chosen by the user)
 -----------------------------------------------------------------
 Prerequisites:
 - filter "python_layerfx_bevel_emboss"
 - gimpressionist presets: "PenDrawing"
                           "Charcoal" 
                           "ColourPencil"
 - patterns: Glasstic_8
             MyGlass_11.jpg
             MyGlass_15.jpg
             Omg1_gray_64.pat
             Omg1_gray_128.pat
 -----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.


'''
#!/usr/bin/env python
from gimpfu import *

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################

def GlassFrameEffectsSE (theOldImage, theOldLayer, GlassEffect, doBevelEmboss, doFlatten) :

    #Initialize an undo, so the process can be undone with a single undo
    pdb.gimp_image_undo_group_start (theOldImage)
    
    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    imageWidth=theOldImage.width
    imageHeight=theOldImage.height
    minSide = min(imageWidth,imageHeight)
    name = theOldLayer.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ", glass-framed "
    
    #check whether there is a selection: 
    nosel = pdb.gimp_selection_is_empty (theOldImage)
    
    if nosel == True:   #if not a selection is forced (the central part)
        selectionWidth=imageWidth*8/10
        selectionHeight=imageHeight*8/10
        cornerRadiusX=imageWidth/20
        cornerRadiusY=imageHeight/20
        pdb.gimp_image_select_round_rectangle (theOldImage, CHANNEL_OP_REPLACE,
                                              (imageWidth-selectionWidth)/2, (imageHeight-selectionHeight)/2,
                                              selectionWidth, selectionHeight, cornerRadiusX, cornerRadiusY)
    #now there is always a selection: save corners
    (thereis, selx1, sely1, selx2, sely2) = pdb.gimp_selection_bounds (theOldImage)
    selectionWidth = selx2 - selx1
    selectionHeight = sely2 - sely1
    borderWidth = (imageWidth-selectionWidth)/2
    borderHeight = (imageHeight-selectionHeight)/2
    minBorder = min(borderWidth,borderHeight)

    pdb.gimp_selection_invert (theOldImage)

    theDuplicatedLayer = pdb.gimp_layer_new_from_drawable (theOldLayer, theOldImage )
    theOldImage.add_layer (theDuplicatedLayer, 0)
    theDuplicatedLayer.name = "Frame Layer"
    pdb.gimp_layer_add_alpha (theDuplicatedLayer)

    if (GlassEffect == 0):  #GlassTile+Blur
        suffix = suffix+"GlassTile+Blur"
        pdb.plug_in_glasstile (theOldImage, theDuplicatedLayer, 50, 50)
        pdb.plug_in_gauss (theOldImage, theDuplicatedLayer, imageWidth/200,  imageHeight/200, 0)    #0=IIR
        
    if (GlassEffect == 1):  #Cartoons
        suffix = suffix+"Cartoons"
        pdb.plug_in_cartoon (theOldImage, theDuplicatedLayer, 15.0, 0.25)   # was 50, 0.5

    if (GlassEffect == 2):  #TexturedGlass+Blur
        suffix = suffix+"TexturedGlass"        
        pdb.plug_in_glasstile (theOldImage, theDuplicatedLayer, 10, 10)
        #pdb.plug_in_gauss (theOldImage, theDuplicatedLayer, imageWidth/200,  imageHeight/200, 0)    #0=IIR
        pdb.plug_in_ripple (theOldImage, theDuplicatedLayer, 5, 5, 0, 0, 1, 1, 1)
        pdb.plug_in_ripple (theOldImage, theDuplicatedLayer, 5, 5, 1, 0, 1, 1, 1)

    if (GlassEffect == 3):  #FractalExplorer
        suffix = suffix+"Fractal"
        theFractalLayer = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "FractalLayer", 100.0, MULTIPLY_MODE)    # was VALUE
        theOldImage.add_layer (theFractalLayer, 0)
        # from Zooming Circle
        pdb.plug_in_fractalexplorer (theOldImage, theFractalLayer, 3,    #3=Barnsley2
                                     0.1875, 0.98438, 0.84375, 1.73438,     #left right top bottom
                                     32,                                    #iterations (was 50)
                                     0.77300, -0.72400,                     #CX,CY
                                   # 1, 0.0, 0.0, 0.0, 0, 0, 0, 0, 0, 0, 256)
                                     0, 1.0, 1.0, 1.0, 2, 1, 1, 0, 0, 0, 256)
        theFractalLayer2 = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "FractalLayer2", 100.0, ADDITION_MODE)   # was GRAIN_MERGE
        theOldImage.add_layer (theFractalLayer2, 0)
        # from The Green Place
        pdb.plug_in_fractalexplorer (theOldImage, theFractalLayer2, 2,    #3=Barnsley1
                                     0.06, 0.95, -0.39, 0.48,              #left right top bottom
                                     50,                                    #iterations
                                    -1.07, -0.20,                         #CX,CY
                                   # 1, 0.0, 0.0, 0.0, 0,0, 0, 0, 0, 0, 256)
                                     0, 1.0, 0.86, 0.02, 0, 1, 1, 1, 1, 0, 256)
        theFractalLayer3 = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "FractalLayer3", 100.0, MULTIPLY_MODE) # was MULTIPLY
        theOldImage.add_layer (theFractalLayer3, 0)
        # from Suns
        pdb.plug_in_fractalexplorer (theOldImage, theFractalLayer3, 1,    #1=Julia
                                    -0.57344, -0.39987, -0.17764, -0.02465,              #left right top bottom
                                     607,                                    #iterations
                                    -0.72400, -0.20000,                         #CX,CY
                                   # 1, 0.0, 0.0, 0.0, 0,0, 0, 0, 0, 0, 256)
                                     0, 1.0, 1.00, 1.00, 2, 0, 1, 0, 0, 0, 256)
        
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, theFractalLayer, CLIP_TO_BOTTOM_LAYER)
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, theFractalLayer2, CLIP_TO_BOTTOM_LAYER)
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, theFractalLayer3, CLIP_TO_BOTTOM_LAYER)
        theDuplicatedLayer.opacity = 75.0
    
    if (GlassEffect == 4):  #Gradient Flare
        suffix = suffix+"GradientFlare"
        pdb.plug_in_gflare (theOldImage, theDuplicatedLayer, "Distant_Sun",
                           int(selectionWidth/2 + selx1), int(selectionHeight/2 + sely1), float(imageWidth/2),
                            0.0, 0.0, 30.0, 1000.0, 0, 3, 0.20)
        pdb.plug_in_nova (theOldImage, theDuplicatedLayer,
                         ((selectionWidth/2) + selx1), ((selectionHeight/2) + sely1),
                         ((200, 200, 0, 255)), 100, 1024, 360) 
        
    if (GlassEffect ==5):  #Polygonize
        suffix = suffix+"Polygonize"
    # adaptive height and space
        tilesize = (imageWidth+imageHeight)/30  # was 20
        neat = 0.20
        height = float((imageWidth+imageHeight)/500.0)
        space = float(height/2.2)
        pdb.gimp_context_set_background ((10,20,30,255))    # was 0,0,0,255
        pdb.gimp_context_set_foreground ((10,20,30,255))    # was 10,10,10,255
        pdb.plug_in_mosaic (theOldImage, theDuplicatedLayer,
                        tilesize, height, space, neat,    # tile characteristics: size, height, space, neat      # was 4,1.0.25
                        True,                        # tile-split
                        135.0,                       # light
                        0.0,                         # colour variation     # was 0.4
                        False, True,                 # antialias, colour averaging      # was True,False
                        3, 0, 1)                     # triangles, surface, grout-colour (0=BG, 1=FG/BG)
        pdb.gimp_brightness_contrast (theDuplicatedLayer, -20, +20)
        theDuplicatedLayer.opacity = 75.0
 
    if (GlassEffect == 6):  #Stained Glass
        suffix = suffix+"StainedGlass"
        tilesize = (imageWidth+imageHeight)/50
        neat = 0.80
        height = float((imageWidth+imageHeight)/500.0)
        space = float(height/2.2)
        pdb.gimp_context_set_background ((10,20,30,255))    # was 0,0,0,255
        pdb.gimp_context_set_foreground ((10,20,30,255))    # was 10,10,10,255
        pdb.plug_in_mosaic (theOldImage, theDuplicatedLayer,
                        tilesize, height, space, neat,    # tile characteristics: size, height, space, neat      # was 4,1.0.25
                        True,                        # tile-split
                        135.0,                       # light
                        0.0,                         # colour variation     # was 0.4
                        False, True,                 # antialias, colour averaging      # was True,False
                        1, 0, 1)                     # hexagons, surface, grout-colour (0=BG, 1=FG/BG)
        thePatternLayer = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "PatternLayer", 50.0, SOFTLIGHT_MODE) # was 100.0
        theOldImage.add_layer (thePatternLayer, 0)
        pdb.gimp_context_set_pattern ("MyGlass_11.jpg")
        pdb.gimp_edit_fill (thePatternLayer, PATTERN_FILL)
        thePatternLayer.mode = HARDLIGHT_MODE
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, thePatternLayer, CLIP_TO_BOTTOM_LAYER)
        pdb.plug_in_erode (theOldImage, theDuplicatedLayer, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
 
    if (GlassEffect == 7):  #Psyco Stained Glass
        suffix = suffix+"PsychoStainedGlass"
        tilesize = (imageWidth+imageHeight)/30
        neat = 0.30
        height = float((imageWidth+imageHeight)/500.0)
        space = float(height/2.2)
        pdb.gimp_context_set_background ((10,20,30,255))    # was 0,0,0,255
        pdb.gimp_context_set_foreground ((10,20,30,255))    # was 10,10,10,255
        pdb.plug_in_mosaic (theOldImage, theDuplicatedLayer,
                        tilesize, height, space, neat,    # tile characteristics: size, height, space, neat      # was 4,1.0.25
                        True,                        # tile-split
                        135.0,                       # light
                        0.0,                         # colour variation     # was 0.4
                        False, True,                 # antialias, colour averaging      # was True,False
                        1, 0, 1)                     # hexagons, surface, grout-colour (0=BG, 1=FG/BG)
        thePatternLayer = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "PatternLayer", 50.0, HARDLIGHT_MODE) # was 100.0
        theOldImage.add_layer (thePatternLayer, 0)
        pdb.gimp_context_set_pattern ("Glasstic_8")
        pdb.gimp_edit_fill (thePatternLayer, PATTERN_FILL)
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, thePatternLayer, CLIP_TO_BOTTOM_LAYER)
        pdb.plug_in_erode (theOldImage, theDuplicatedLayer, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
        thePlasmaLayer = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "PlasmaLayer", 50.0, HARDLIGHT_MODE)  # was 100.0 HARDLIGHT
        theOldImage.add_layer (thePlasmaLayer, 0)
        pdb.plug_in_plasma (theOldImage, thePlasmaLayer, 0, 1.0)
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, thePlasmaLayer, CLIP_TO_BOTTOM_LAYER)
        pdb.plug_in_erode (theOldImage, theDuplicatedLayer, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128) # added
        theDuplicatedLayer.opacity = 75.0
    
    if (GlassEffect == 8):  #Etching
        suffix = suffix+"Etching"
        Preset = "PenDrawing"
        pdb.plug_in_gimpressionist (theOldImage, theDuplicatedLayer, Preset)
        pdb.gimp_brightness_contrast (theDuplicatedLayer, -20, +20)
    
    if (GlassEffect == 9):  #Charcoal
        suffix = suffix+"Charcoal"
        pdb.gimp_desaturate (theDuplicatedLayer)
        Preset = "Charcoal"
        pdb.plug_in_gimpressionist (theOldImage, theDuplicatedLayer, Preset)
        pdb.gimp_brightness_contrast (theDuplicatedLayer, -15, +15)
    
    if (GlassEffect == 10):  #ColourPencil
        suffix = suffix+"ColourPencil"
        pdb.gimp_hue_saturation (theDuplicatedLayer, 0, 0, 0, +100)
        Preset = "ColourPencil"
        pdb.plug_in_gimpressionist (theOldImage, theDuplicatedLayer, Preset)
        pdb.gimp_brightness_contrast (theDuplicatedLayer, -20, +20)
    
    if (GlassEffect == 11):  #OMG GlassBrick
        suffix = suffix+"OMGGlassBrick"
        thePatternLayer = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "PatternLayer", 100.0, HARDLIGHT_MODE)
        theOldImage.add_layer (thePatternLayer, 0)
        if minBorder < 128:
            pdb.gimp_context_set_pattern ("Omg1_gray_64.pat")
        else:
            pdb.gimp_context_set_pattern ("Omg1_gray_128.pat")
        pdb.gimp_edit_fill (thePatternLayer, PATTERN_FILL)
        theDuplicatedLayer = pdb.gimp_image_merge_down (theOldImage, thePatternLayer, CLIP_TO_BOTTOM_LAYER)
    
    pdb.gimp_selection_invert (theOldImage) #restore user selection
        
    theNewLayer = pdb.gimp_layer_new_from_drawable (theOldLayer, theOldImage) 
    theOldImage.add_layer (theNewLayer, 0)                    
    theNewLayer.name = "Glass Layer"
    theNewLayer.mode = OVERLAY_MODE
    theNewLayer.opacity = 90.0
    pdb.gimp_layer_add_alpha (theNewLayer)
   
    pdb.gimp_edit_cut (theNewLayer)    #keep only the outside part of the image
    pdb.gimp_selection_invert (theOldImage)  #frame area (outside)
   
    #use of average colour -----------------------------------------------------
    (meanR, stddev, median, pixels, count, percent)=pdb.gimp_histogram(theNewLayer, 1, 0, 255)
    (meanG, stddev, median, pixels, count, percent)=pdb.gimp_histogram(theNewLayer, 2, 0, 255)
    (meanB, stddev, median, pixels, count, percent)=pdb.gimp_histogram(theNewLayer, 3, 0, 255)
    avg_colour = (int(meanR), int(meanG), int(meanB), 255)
    #info_msg ("R,G,B="+str(int(meanR))+","+str(int(meanG))+","+str(int(meanB)))
    pdb.gimp_context_set_foreground ((avg_colour))
    pdb.gimp_edit_fill (theNewLayer, FOREGROUND_FILL)
    pdb.gimp_hue_saturation (theNewLayer, 0, 0, 0, 100)
    thePatternLayer = pdb.gimp_layer_new (theOldImage, imageWidth, imageHeight, RGBA_IMAGE,
                                              "PatternLayer", 100.0, GRAIN_MERGE_MODE)
    theOldImage.add_layer (thePatternLayer, 0)
    pdb.gimp_context_set_pattern ("MyGlass_15.jpg")
    pdb.gimp_edit_fill (thePatternLayer, PATTERN_FILL)
    if (GlassEffect == 4):  #Gradient Flare, reduce impact
        thePatternLayer.opacity = 50.0
    if (GlassEffect == 1):  #Cartoons, reduce impact
        thePatternLayer.mode = SOFTLIGHT_MODE
    if (GlassEffect ==5):  #Polygonize, reduce impact
        thePatternLayer.opacity = 50.0
    theNewLayer = pdb.gimp_image_merge_down (theOldImage, thePatternLayer, CLIP_TO_BOTTOM_LAYER)
    if (GlassEffect == 9):  #Charcoal
        pdb.gimp_desaturate (theNewLayer)
    if (GlassEffect ==5):  #Polygonize, brighten
        theNewLayer.mode = DODGE_MODE
    elif (GlassEffect == 4):  #Gradient Flare, increase effect
        theNewLayer.mode = SATURATION_MODE
    else:    
        theNewLayer.mode = OVERLAY_MODE

    #end average colour --------------------------------------------------------

    if (doBevelEmboss == True):
        theTopLayer = pdb.gimp_layer_new_from_drawable (theOldLayer, theOldImage) 
        theOldImage.add_layer (theTopLayer, 0)                    
        theTopLayer.name = "Emboss Layer"
        theTopLayer.mode = NORMAL_MODE
        theTopLayer.opacity = 100.0
        pdb.gimp_layer_add_alpha (theTopLayer)
        pdb.gimp_edit_cut (theTopLayer)    #keep only the outside part of the image

        bevelDepth = int(imageWidth/50)
        if (bevelDepth > 64):	 #added check to prevent plug-in error
            bevelDepth = 64 
        bevelSize = int(imageWidth/100)
        pdb.python_layerfx_bevel_emboss (theOldImage, theTopLayer, 
                                 1,               #Style = Inner Bevel
                                 bevelDepth,      #Depth 
                                 1,               #Direction = Down
                                 bevelSize,       #Size 
                                 2,               #Soften 
                                 120,             #Angle 
                                 30,              #Altitude 
                                 0,               #Gloss Contour 
                                 (223, 255, 239, 255),   #Highlight Color 
                                 GRAIN_MERGE_MODE,        #Highlight Mode = Hue Mode
                                 100,             #Highlight Opacity 
                                 (4, 16, 8, 255),        #Shadow Color 
                                 GRAIN_MERGE_MODE,        #Shadow Mode = Hue Mode
                                 100,             #Shadow Opacity 
                                 0,               #Surface Contour
                                 False,           #Use Texture
                                 "Gold 322",      #Texture Pattern
                                 1.0,             #Texture Scale
                                 1.0,             #Texture Depth 
                                 False,           #Invert 
                                 True)            #Merge with layer
        
    if (doFlatten == True) :
        Flat_Layer = pdb.gimp_image_flatten(theOldImage)
        Flat_Layer.name = name+suffix

    pdb.gimp_selection_none (theOldImage)  

    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)

    pdb.gimp_image_undo_group_end (theOldImage)
    pdb.gimp_displays_flush()
    
    if (doFlatten == True) :
        return(Flat_Layer)
    else:
        return()
        
# This is the plugin registration function
register(
         "GlassFrameEffectsSE",
         "to create a glass frame around an image (Special Edition)",
         "This filter creates a frame around an image with a simple or sophisticated glass effect (Special Edition)",
         "Diego",
         "Diego Nassetti",
         "December 2016",
         "GlassFrameEffectsSE",
         "RGB*", 
        [
            (PF_IMAGE, "image", "Input image", None),
            (PF_DRAWABLE, "drawable", "Input drawable", None),
            (PF_OPTION, "effect", "Effect on Glass Layer", 0, ("GlassTile&Blur",
                                                               "Cartoons",
                                                               "Textured Glass",
                                                               "Fractalize",
                                                               "Gradient Flare",
                                                               "Polygonize",
                                                               "Stained Glass",
                                                               "Psychedelic Glass",
                                                               "Etching",
                                                               "Charcoal",
                                                               "Coloured Pencil",
                                                               "OMG Glass Bricks")),
            (PF_TOGGLE, "bevel", "Bevel/Emboss Selections", True),
            (PF_TOGGLE, "flatten", "Flatten Image", False),
        ], 
        [
          (PF_DRAWABLE, "odrawable", "Output drawable", None),
        ],
        GlassFrameEffectsSE,
        menu="<Image>/Diego/Frames",
        )

main()
 
